//-*-C++-*- Header file for the QMeter Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _QMeter_h_
#define _QMeter_h_ 1
#include <qwidget.h>
#include <qlabel.h>
#include <qpixmap.h>



/** 
  The {\bf QMeter} object is a special {\bf QLabel}.  It displays a
  rectangular, vertical LCD-type digital meter.  If you make it large
  enough, a {\bf QMeter} will be composed of several individual
  "lights" stacked together, as in an LCD-meter. If the {\bf QMeter}
  is too small, it will be continuous.

  The {\bf QMeter} changes level continuously.  That is, as you
  increase the level, each "light" grows in size.  After one light is
  completely lit, the next one is drawn.  There is no set level
  cooresponding to a particular change in the lights.  That depends on
  the size of the meter, the size of the lights, and so on.

  This class inherits the Qt {\bf QLabel} class.  I've documented the
  protected methods in case you want to use this class as a base for a
  new widget.

  {\bf
  \begin{verbatim}
  
  ----------------------------------------------------------
  ----------------------------------------------------------
  \end{verbatim}
  }

  @memo LCD-type digital meter.
  */
class QMeter : public QLabel
{
	 //
	 Q_OBJECT

   public:
	 /**
	   Class constructor.  The arguments are the same as those for
	   a {\bf QLabel}.
	   @memo
	   */
	 QMeter( QWidget* parent=0, const char* name=0, WFlags f=0 ); 

	 /**
	   Redraws the meter.  You need to make a call to
	   \Ref{initMeter}() if you've changed anything other than the
	   meter level.
	   @see initMeter
	   @memo
	   */
	 void Redraw(void);

	 /**
	   Sets the size of the "lights" in the meter, in terms of
	   percent of the total size of the meter.  In some sense, it
	   also sets the number of lights in the meter.  E.g.:
	   "setLightSize(0.05)" makes the meter have 20 "lights".

	   Automatically calls \Ref{initMeter}(), but {\em not}
	   \Ref{Redraw}().
	   @see initMeter, Redraw
	   @memo
	   */
	 void setLightSize(float percent) {
	   lightsize = percent;
	   initMeter();
	   setLevel(current_level);
	 }

	 /**
	   Returns the level, in units of percent of the total meter
	   size, that the meter is currently set to.
	   @memo
	   */
	 float getLevel(void) {
	   return (current_level);
	 }

	 /**
	   Initializes the meter.

	   You should call this after any change in size.  It's
	   automatically called by \Ref{setLightSize}() and the class
	   constructor.  It does not, however, call \Ref{Redraw}(); you
	   need to do that yourself.
	   @see Redraw
	   @memo
	   */
	 void initMeter(void) {
	   int nuh,nl,ht,wd;
	   float fppl;
	   

	   if(lightsize < 0.0)
		 lightsize *= -1;

	   if(lightsize > 1.0)
		 lightsize = 1.0;

	   ht = height();
	   wd = width();
	   if(oldSize.height() != ht)
		 {
		 if(ht < 4)
		    ht = 4;
		 ht -= 2;
		 }
	   if(oldSize.width() != wd)
		 {
		 if(wd < 4)
		    wd = 4;
		 wd -= 2;
		 }

	   fppl = (lightsize*(float)ht);
	   if(fppl < 1.0)
		 {
		 ppl = 1;
		 continuous=true;
		 }
	   else
		 {
		 continuous=false;
		 ppl = (int)fppl;
		 if( (fppl-(float)ppl)>=0.5 )
		    ppl++;
		 }
	   
	   if(ppl != 1)
		 {
		 nl = ht/ppl;
		 if(nl>0)
		    nuh = nl*ppl;
		 else
		    {
		    nuh = ht;
		    continuous=true;
		    }
		 }
	   else
		 {
		 nuh = ht;
		 continuous=true;
		 }
		 
	   meter_map.resize( wd, nuh );
	   oldSize = size();
	 }


   public slots:
	 /**
	   Sets the meter level to {\em percent}, which is the percent of
	   the total meter size.  Automatically calls \Ref{Redraw}().
	   @see Redraw
	   @memo
	   */
	 void setLevel(double percent);
	 /**
	   Overloaded version of \Ref{setLevel}({\em float}).
	   Essentially a call to {\tt setLevel((float)ipercent/100.0)}.
	   @see Redraw
	   @memo
	   */
	 void setLevel(int ipercent);

   protected:
	 /**
	   The actual {\bf QPixmap} of the meter gets stored here.
	   @memo
	   */
	 QPixmap meter_map;
	 /**
	   Stores the light size set by \Ref{setLightSize}().
	   @memo
	   */
	 float lightsize;
	 /**
	   Stores the meter level.
	   @memo
	   */
	 float current_level;
	 /**
	   Contains the height of each "light" in pixels.
	   @memo
	   */
	 int ppl;
	 /**
	   Used internally by \Ref{initMeter}().
	   @memo
	   */
	 QSize oldSize;
	 /**
	   Set by \Ref{initMeter}(); indicates whether the {\bf QMeter}
	   should be continuous or subdivided into "lights."
	   @memo
	   */
	 bool continuous;

	 /**
	   This method is used internally by \Ref{Redraw}() to perform
	   certain coordinate transforms.
	   @memo
	   */
	 QRect flipVert(int x0, int y0, int ww, int hh, 
				 QPixmap map) {
	   int h0;
	   h0=map.height();
	   return(QRect(x0,h0-hh-y0,ww,hh));
	 }
};

#endif // _QMeter_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
